#include	<stdio.h>
#include	<fcntl.h>
#include	<sys/types.h>
#include	<sys/file.h>
#include	<sys/mman.h>
#include        <sys/ioctl.h>
#include        <sys/errno.h>
#include        <sbusdev/vme.h>
#include        <sbusdev/vmelib.h>

#define         SLOT 6
/* SLOT = 0 for VME_A16D16,
   SLOT = 1 for VME_A16D32,
   SLOT = 2 for VME_A24D16,
   SLOT = 3 for VME_A24D32,
   SLOT = 4 for VME_A32D16,
   SLOT = 5 for VME_A32D32,
*/
#define VME_A16BASEADDR         0
#define VME_A24BASEADDR         0
#define VME_A32BASEADDR         0
#define VME_A16TOPADDR     0xFFFF   /*actually  0xFFFFFFFF ,but don't care */
#define VME_A24TOPADDR   0xFEFFFF   /*actually  0xFFFEFFFF ,but don't care */
#define VME_A32TOPADDR 0xFEFFFFFF   /*actually  0xFEFFFFFF ,but don't care */


extern	int	errno;
int MA_A32SET();

/* For VME devices */
static int vme_fd[SLOT];        /*  for open() */ 
static int vme_offset[SLOT];    /*  =vmeaddr   */
static int vme_size[SLOT];      /* size + vme_offset[slot]%vme_pagesize; */


/* For Interrupt handler */
int     path_vme;
int     path_int;
static struct intcmd intcmd;

caddr_t vme_mapopen(mode, vmeaddr, size)
int mode;
caddr_t vmeaddr;
size_t size;
{
  caddr_t vme_vaddr;
  int slot;
  char vme_dev[15];
  int vme_mapsize;      /* for my environment 0x1000  */
  int vme_pagesize;
  u_char a32map;

  errno = 0;
  vme_pagesize = getpagesize();
  switch (mode) 
    {
    case VME_A16D16MODE :
      slot = 0;
      if( (vmeaddr < (caddr_t)VME_A16BASEADDR) || 
	 ((vmeaddr+size) > (caddr_t)(VME_A16TOPADDR)) )
	return (caddr_t)-1;
      vme_offset[slot] = (int)vmeaddr;
      strcpy(vme_dev,"/dev/vme16d16");
      a32map=0xf;
      break;
    case VME_A16D32MODE :
      slot = 1;
      if( (vmeaddr < (caddr_t)VME_A16BASEADDR) || 
	 ((vmeaddr+size) > (caddr_t)(VME_A16TOPADDR)) )
	return (caddr_t)-1;
      vme_offset[slot] = (int)vmeaddr;
      strcpy(vme_dev,"/dev/vme16d32");
      a32map=0xf;
      break;
    case VME_A24D16MODE :
      slot = 2;
      if( (vmeaddr < (caddr_t)VME_A24BASEADDR) || 
	 ((vmeaddr+size) > (caddr_t)(VME_A24TOPADDR)) )
	return (caddr_t)-1;
      vme_offset[slot] = (int)vmeaddr;
      strcpy(vme_dev,"/dev/vme24d16");
      a32map=0xf;
      break;
    case VME_A24D32MODE :
      slot = 3;
      if( (vmeaddr < (caddr_t)VME_A24BASEADDR) || 
	 ((vmeaddr+size) > (caddr_t)(VME_A24TOPADDR)) )
	return (caddr_t)-1;
      vme_offset[slot] = (int)vmeaddr;
      strcpy(vme_dev,"/dev/vme24d32");
      a32map=0xf;
      break;
    case VME_A32D16MODE :
      slot = 4;
      if( (vmeaddr < (caddr_t)VME_A32BASEADDR) || 
	 ((vmeaddr+size) > (caddr_t)(VME_A32TOPADDR)) )
	return (caddr_t)-1;
      vme_offset[slot] = (int)vmeaddr;
      strcpy(vme_dev,"/dev/vme32d16");
      a32map=(u_char)vmeaddr>>28;
      break;
    case VME_A32D32MODE :
      slot = 5;
      if( (vmeaddr < (caddr_t)VME_A32BASEADDR) || 
	 ((vmeaddr+size) > (caddr_t)(VME_A32TOPADDR)) )
	return (caddr_t)-1;
      strcpy(vme_dev,"/dev/vme32d32");
      a32map=(u_char)vmeaddr>>28;
      break;
    };
  vme_offset[slot] = (int)vmeaddr;  
  vme_mapsize = size + (int)vmeaddr%vme_pagesize;
  vme_size[slot] = vme_mapsize;
  if ((vme_fd[slot] = open(vme_dev, O_RDWR)) == -1)
    return (caddr_t)-1;
  /* if mmap fails, return value becomes -1 */

  MA_A32SET(a32map,vme_fd[slot]);

  vme_vaddr = (caddr_t)mmap(0, vme_mapsize, PROT_READ | PROT_WRITE,
			    MAP_SHARED, vme_fd[slot],
			    (vme_offset[slot]/vme_pagesize)*vme_pagesize );
  if (vme_vaddr == (caddr_t) -1) {
    return (caddr_t)-1;
  } else
    return (vme_vaddr + vme_offset[slot]%vme_pagesize);
}



caddr_t vme_mapclose(addr, mode)
     caddr_t addr;
     int mode;
{
  int status, vme_vaddr, slot;
  
  errno = 0;
  switch (mode) {
  case VME_A16D16MODE :
    slot = 0;
    break;
  case VME_A16D32MODE :
    slot = 1;
    break;
  case VME_A24D16MODE :
    slot = 2;
    break;
  case VME_A24D32MODE :
    slot = 3;
    break;
  case VME_A32D16MODE :
    slot = 4;
    break;
  case VME_A32D32MODE :
    slot = 5;
    break;
  }
  vme_vaddr = (int)addr - vme_offset[slot]%getpagesize(); /*same as when open*/
  status = munmap(vme_vaddr, vme_size[slot]);
  if( status == -1 ) {
    return (caddr_t)-1;
  }
  close(vme_fd[slot]);
  return 0;
}



int vme_mapprobe( mm, size, mode )
     char *mm;
     int size, mode;
{
  int i, temp, *ptrl;
  short *ptrw;
  char *ptrc;
  
  switch ( mode )  {
  case VME_A16D32MODE :
  case VME_A32D32MODE :
  case VME_A24D32MODE :
    ptrl = ( long *)mm;
    for( i = 0; i < size/sizeof(long); i++ )
      temp = *ptrl++;
  case VME_A16D16MODE :
  case VME_A24D16MODE :
  case VME_A32D16MODE :
    ptrw = ( short *)mm;
    for( i = 0; i < size/sizeof(short); i++ )
      temp = *ptrw++;
    ptrc = ( char *)mm;
    for( i = 0; i < size/sizeof(long); i++ )
      temp = *ptrc++;
    break;
  }
}



int vme_intopen( vector, intrno )
     int vector;
     int *intrno;
{
  
  errno = 0;
  if ((path_int = open("/dev/int", O_RDWR)) == -1){
    printf("vmelib.c  : open error \n");
    return errno;
  }
  intcmd.number = vector;
  if( (ioctl(path_int, INT_OPEN, &intcmd )) == -1 ){
    printf("vmelib.c  : ioctl(OPEN) error \n");
    return errno;
  }
  *intrno = intcmd.reply1;
  return 0;
}

int vme_intclose( intrno )
     int intrno;
{
  
  errno = 0;
  intcmd.number = intrno;
  if( ( ioctl( path_int, INT_CLOSE, &intcmd ) ) == -1 )
    return errno;
  return 0;
}


int vme_intenable( intrno )
     int intrno; 
{
  intcmd.number = intrno;
  errno = 0;
  if( ( ioctl( path_int, INT_ENABLE, &intcmd ) ) == -1 )
    return errno;
  return 0;
}

int vme_intdisable( intrno )
     int intrno;
{
  intcmd.number=intrno;
  errno = 0;
  if( ( ioctl( path_int, INT_DISABLE, &intcmd ) ) == -1 )
    return errno;
  return 0;
}

int vme_intwait( intrno, interrupt_count, timeout )
int intrno, *interrupt_count, timeout; 
{
  
  errno = 0;
  intcmd.number = intrno;
  intcmd.data = timeout;
  ioctl( path_int, INT_WAIT, &intcmd );
  *interrupt_count = intcmd.reply1;
  if( errno == EINTR && intcmd.reply2 == MA_TIMEOUT )
    return ETIMEDOUT;
  if( errno == EINTR && intcmd.reply2 == MA_SIGNAL_STOP )
    return errno;
  if( errno == EINTR && intcmd.reply2 == 0 )
    return 0;
  return errno;
}

int vme_intnowait( intrno, interrupt_count )
     int intrno;
     int *interrupt_count;
{
  
  errno = 0;
  intcmd.number = intrno;
  ioctl( path_int, INT_NOWAIT, &intcmd );
  *interrupt_count = intcmd.reply1;
  if( errno == EINTR && intcmd.reply2 == 0 )
    return 0;
  return errno;
}

int vme_intclear( intrno )
     int intrno;
{
  intcmd.number = intrno;
  if( ( ioctl( path_int, INT_CLEAR, &intcmd ) ) == -1 )
    return errno;
}

int vme_intcheck( intrno, interrupt_count )
     int intrno, *interrupt_count; 
{
  errno = 0;
  intcmd.number = intrno;
  ioctl(path_int, INT_CHECK, &intcmd );
  *interrupt_count = intcmd.reply1;
  if( errno == EINTR && intcmd.reply2 == 0 )
    return 0;
  return errno;
}

MA_A32SET(a32map,path_vme)
     u_char a32map;
     int path_vme;
{
  struct vme_ioctl vme_ioctl; 
  u_char a32map_old;

  /* mmap() will cut of top 4 bits (supplied by a32map). Round to page */  
  /* Get a32map register and save */
  vme_ioctl.reg_no = A32MAP_REG;
  if ((ioctl(path_vme, VME_GET_REG, &vme_ioctl)) == -1) {
    printf("ioctl VME_GET_REG failed \n");
  }
  a32map = ((u_char)vme_ioctl.reg_data & (u_char)0xf0) + a32map;
  vme_ioctl.reg_no = A32MAP_REG;
  vme_ioctl.reg_data = (u_long)a32map;
  if ((ioctl(path_vme, VME_SET_REG, &vme_ioctl)) == -1) {
    printf("ioctl VME_SET_REG failed \n");
  }
}				

MAMA_A32SET(vmeaddr)
     addr_t vmeaddr;
{
  struct vme_ioctl vme_ioctl; 
  u_char a32map_old;
  u_char a32map;
  int dummyfd ;

  if ((dummyfd= open("/dev/vme32d32", O_RDWR)) == -1){
    printf("vmelibXXX\n");
  }
  printf("dummyfd=%d",dummyfd);
  vme_ioctl.reg_no = A32MAP_REG;
  if ((ioctl(dummyfd, VME_GET_REG, &vme_ioctl)) == -1) {
    printf(" vmelib.c :ioctl VME_GET_REG failed q \n");
  }

  a32map=(u_char)vmeaddr>>28;

  a32map = ((u_char)vme_ioctl.reg_data & (u_char)0xf0) + a32map;
  vme_ioctl.reg_no = A32MAP_REG;
  vme_ioctl.reg_data = (u_long)a32map;
  if ((ioctl(dummyfd, VME_SET_REG, &vme_ioctl)) == -1) {
    printf("vmelib.c :ioctl VME_SET_REG failed \n");
  }
}




